import logging
from pprint import pformat

from tgclients.metadata import TextgridMetadata
from wsgidav.util import join_uri

from repdav.textgrid_dav_provider import (
    TextgridAggregation,
    TextgridProject,
    TextgridResource,
    TextgridResourceProvider,
    TextgridRoot,
)

_logger = logging.getLogger(__name__)

# ============================================================================
# TextgridNamedResourceProvider
# ============================================================================
class TextgridNamedResourceProvider(TextgridResourceProvider):
    """DAV provider that serves Textgrid resources."""

    def get_resource_inst(self, path, environ):
        _logger.debug(
            "Called TextgridNamedResourceProvider.get_resource_inst(self, %s, %s).",
            path,
            pformat(environ),
        )
        self._count_get_resource_inst += 1
        root = TextgridNamedRoot("/", environ)
        # an instance of _DAVResource (i.e. either DAVCollection or DAVNonCollection)
        return root.resolve("/", path)


class TextgridNamedRoot(TextgridRoot):
    """Top level collection that incorporates Textgrid projects.

    This is implemented as READ-ONLY as projects may not be created with webDAV.
    """

    def __init__(self, path, environ):
        super().__init__(path, environ)

    def get_member_names(self):
        _logger.debug("Called TextgridNamedRoot.get_member_names(self).")
        projects = []
        for project in tuple(self._auth.list_assigned_projects(self._sid)):
            projects.append(
                self._auth.get_project_description(project).name + " [" + project + "]"
            )
        _logger.debug("MY PROJECTS: %s", projects)
        return projects

    def get_member(self, name):
        _logger.debug("Called TextgridRoot.get_member(self, %s).", name)
        return TextgridNamedProject(join_uri(self.path, name), self.environ)


class TextgridNamedProject(TextgridProject):
    def __init__(self, path, environ):
        _logger.debug("Called TextgridNamedProject.__init__(self, %s, environ).", path)
        super().__init__(path, environ)
        self._tgmeta = TextgridMetadata()
        name = self.path.split("/")[-1]
        # the projectID is found between square brackets at the end of the name string
        self._project_id = name[name.rfind("[") + 1 : name.rfind("]")]

    def get_member_names(self):
        _logger.debug("Called TextgridNamedProject.get_member_names(self).")
        response = self._tgsearch.list_project_root(self._project_id, self._sid)
        names = []
        for result in response.result:
            names.append(self._tgmeta.filename_from_metadata(result))
        return names

    def get_member(self, name):
        _logger.debug("Called TextgridNamedProject.get_member(self, %s).", name)
        tguri = "textgrid:" + self._tgmeta.id_from_filename(name)
        response = self._tgsearch.info(tguri, self._sid)
        info = {
            name: {
                "title": response.result[0].object_value.generic.provided.title,
                "format": response.result[0].object_value.generic.provided.format,
                "extent": response.result[0].object_value.generic.generated.extent,
            }
        }
        _logger.info("INFO: %s", info)
        if "aggregation" in info[name]["format"]:
            return TextgridNamedAggregation(
                join_uri(self.path, name), self.environ, info, self._tgmeta
            )
        return TextgridNamedResource(
            join_uri(self.path, name), self.environ, info, self._tgmeta
        )


class TextgridNamedAggregation(TextgridAggregation):
    def __init__(self, path, environ, info, tgmeta):
        _logger.debug(
            "Called TextgridNamedAggregation.__init__(self, %s, environ, info).", path
        )
        super().__init__(path, environ, info)
        self._tgmeta = tgmeta
        self._tguri = "textgrid:" + self._tgmeta.id_from_filename(self.path.split("/")[-1])

    def get_member_names(self):
        _logger.debug("Called TextgridNamedAggregation.get_member_names(self).")
        response = self._tgsearch.list_aggregation(self._tguri, self._sid)
        names = []
        for result in response.result:
            names.append(self._tgmeta.filename_from_metadata(result))
        return names

    def get_member(self, name):
        _logger.debug("Called TextgridNamedAggregation.get_member(self, %s).", name)
        tguri = "textgrid:" + self._tgmeta.id_from_filename(name)
        response = self._tgsearch.info(tguri, self._sid)
        info = {
            name: {
                "title": response.result[0].object_value.generic.provided.title,
                "format": response.result[0].object_value.generic.provided.format,
                "extent": response.result[0].object_value.generic.generated.extent,
            }
        }
        _logger.info("INFO: %s", info)
        if "aggregation" in info[name]["format"]:
            return TextgridNamedAggregation(
                join_uri(self.path, name), self.environ, info, self._tgmeta
            )
        return TextgridNamedResource(
            join_uri(self.path, name), self.environ, info, self._tgmeta
        )


class TextgridNamedResource(TextgridResource):
    """Non-Aggregation resources."""

    def __init__(self, path, environ, info, tgmeta):
        _logger.debug("Called TextgridNamedResource.__init__(self, %s, environ).", path)
        super().__init__(path, environ, info)
        self._tgmeta = tgmeta
        self._tguri = "textgrid:" + self._tgmeta.id_from_filename(self.path.split("/")[-1])
